@@ -5,13 +5,13 @@ use warnings;
 use App::cpm::Master;
 use App::cpm::Worker;
 use App::cpm::Logger;
+use App::cpm::version;
 use Getopt::Long qw(:config no_auto_abbrev no_ignore_case bundling);
 use Pod::Usage ();
 use Cwd 'abs_path';
 use Config;
-use version;
 
-our $VERSION = '0.113';
+our $VERSION = '0.114';
 
 sub new {
     my ($class, %option) = @_;
@@ -42,6 +42,8 @@ sub parse_options {
         "w|workers=i" => \($self->{workers}),
         "target-perl=s" => \my $target_perl,
         "test!" => sub { $self->{notest} = $_[1] ? 0 : 1 },
+        "cpanfile=s" => \($self->{cpanfile}),
+        "snapshot=s" => \($self->{snapshot}),
     or exit 1;
 
     $self->{local_lib} = abs_path $self->{local_lib} unless $self->{global};
@@ -50,7 +52,10 @@ sub parse_options {
     if ($target_perl) {
         # 5.8 is interpreted as 5.800, fix it
         $target_perl = "v$target_perl" if $target_perl =~ /^5\.[1-9]\d+$/;
-        $self->{target_perl} = version->parse($target_perl)->numify;
+        $self->{target_perl} = App::cpm::version->parse($target_perl)->numify;
+        if ($self->{target_perl} > $]) {
+            die "--target-perl must be lower than your perl version $]\n";
+        }
     }
 
     $App::cpm::Logger::COLOR = 1 if $self->{color};
@@ -144,8 +149,18 @@ sub cmd_install {
         $worker->run_loop;
     };
 
-    my @package = map +{package => $_, version => 0}, @argv;
-    if (!@package && -f $self->{cpanfile}) {
+    my $git;
+    my @package;
+    for my $arg (@argv) {
+        if ($arg =~ /(?:^git:|\.git(?:@.+)?$)/) {
+            my $dist = App::cpm::Distribution->new(distfile => $arg, provides => []);
+            $master->add_distribution($dist);
+            $git++;
+        } else {
+            push @package, {package => $arg, version => 0};
+        }
+    }
+    if (!$git && !@package && -f $self->{cpanfile}) {
         warn "Loading modules from $self->{cpanfile}...\n";
         @package = grep {
             !$master->is_core($_->{package}, $_->{version})
@@ -245,9 +260,12 @@ App::cpm - a fast cpan module installer
 
 =head1 DESCRIPTION
 
+=for html
+<a href="https://raw.githubusercontent.com/skaji/cpm/master/xt/demo.gif"><img src="https://raw.githubusercontent.com/skaji/cpm/master/xt/demo.gif" alt="demo" style="max-width:100%;"></a>
+
 B<THIS IS EXPERIMETNAL.>
 
-cpm is a fast cpan module installer, which uses L<Menlo::CLI::Compat> in parallel.
+cpm is a fast cpan module installer, which uses L<Menlo> in parallel.
 
 =head1 MOTIVATION
 
@@ -260,7 +278,7 @@ then it takes quite a lot of time to install them.
 
 So my motivation is simple: I want to install cpan modules as fast as possible.
 
-=head1 HOW FAST?
+=head2 HOW FAST?
 
 Just an example:
 
@@ -272,6 +290,23 @@ Just an example:
 
 This shows cpm is 3x faster than cpanm.
 
+=head1 ROADMAP
+
+If you all find cpm useful,
+then cpm should be merged into cpanm 2.0. How exciting!
+
+To merge cpm into cpanm, there are several TODOs:
+
+=over 4
+
+=item * Win32? - support platforms that do not have fork(2) system call
+
+=item * Logging? - the parallel feature makes log really messy
+
+=back
+
+Your feedback is highly appreciated.
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2015 Shoichi Kaji E<lt>skaji@cpan.orgE<gt>
@@ -281,6 +316,9 @@ it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
+
+L<Perl Advent Calendar 2015|http://www.perladvent.org/2015/2015-12-02.html>
+
 L<App::cpanminus>
 
 L<Menlo>
